# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2022 John Mille<john@compose-x.io>

"""
Manages certbot certificates download from certbot-aws-store
"""

from __future__ import annotations

import os
from typing import TYPE_CHECKING

from boto3.session import Session

if TYPE_CHECKING:
    from .input import Model

from os import makedirs, path

from certbot_aws_store.certificate import AcmeCertificate

from ecs_files_composer.common import LOG


def process_certbot_aws_store_certificates(job: Model) -> None:
    """
    Pulls certificates from certbot-aws-store to local filesystem
    If the path does not exist, creates a new directory for download.
    """
    if not job.certbot_store:
        return
    default_table_name = job.certbot_store.certificates_registry_table_name
    if job.certbot_store.certificates_registry_table_region:
        default_table_region = job.certbot_store.certificates_registry_table_region
    else:
        default_table_region = os.getenv("AWS_DEFAULT_REGION", Session().region_name)
    for _hostname, _definition in job.certbot_store.certificates.items():
        certificate = AcmeCertificate(
            _hostname,
            None,
            table_name=_definition.certificates_registry_table_name
            if _definition.certificates_registry_table_name
            else default_table_name,
            region_name=_definition.certificates_registry_table_region
            if _definition.certificates_registry_table_region
            else default_table_region,
        )
        if not path.exists(_definition.storage_path):
            makedirs(_definition.storage_path, exist_ok=True)
        try:
            certificate.pull(_definition.storage_path)
            LOG.info("Successfully pulled certificates for %s", _hostname)
        except Exception as error:
            LOG.exception(error)
            LOG.error(
                "Failed to download certificate from certbot-aws-store", _hostname
            )



# generated by datamodel-codegen:
#   filename:  ecs-files-input.json
#   timestamp: 2023-02-28T08:48:36+00:00

from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Optional, Union

from pydantic import AnyUrl, BaseModel, EmailStr, Extra, Field, constr


class S3Uri(BaseModel):
    __root__: str = Field(
        ...,
        description="s3://bucket-name/path/to/file simple syntax. Does not support IamOverride",
        regex="^s3://([a-zA-Z\\d\\-.]+)/([\\S]+)$",
    )


class ComposeXUri(BaseModel):
    __root__: str = Field(
        ...,
        description="bucket_name::path/to/file format used in other compose-x projects",
        regex="([a-zA-Z\\-\\d.]+)::([\\S]+)$",
    )


class Encoding(Enum):
    base64 = "base64"
    plain = "plain"


class Context(Enum):
    plain = "plain"
    jinja2 = "jinja2"


class IgnoreFailureItem(BaseModel):
    class Config:
        extra = Extra.forbid

    commands: Optional[bool] = Field(
        False, description="Ignore if any of the commands failed"
    )
    mode: Optional[bool] = Field(
        False, description="Ignore if `mode` (using chmod) failed."
    )
    owner: Optional[bool] = Field(
        False, description="Ignore if `owner` (using chown) failed"
    )
    source_download: Optional[bool] = Field(
        False,
        description="Ignore if a Source download failed. Any subsequent action is cancelled.",
    )


class UrlDef(BaseModel):
    url: Optional[AnyUrl] = Field(None, alias="Url")
    username: Optional[str] = Field(None, alias="Username")
    password: Optional[str] = Field(None, alias="Password")


class IamOverrideDef(BaseModel):
    role_arn: Optional[str] = Field(None, alias="RoleArn")
    session_name: Optional[str] = Field(
        "S3File@EcsConfigComposer",
        alias="SessionName",
        description="Name of the IAM session",
    )
    external_id: Optional[str] = Field(
        None,
        alias="ExternalId",
        description="The External ID to use when using sts:AssumeRole",
    )
    region_name: Optional[str] = Field(None, alias="RegionName")
    access_key_id: Optional[str] = Field(
        None, alias="AccessKeyId", description="AWS Access Key Id to use for session"
    )
    secret_access_key: Optional[str] = Field(
        None, alias="SecretAccessKey", description="AWS Secret Key to use for session"
    )
    session_token: Optional[str] = Field(None, alias="SessionToken")


class CommandsDef(BaseModel):
    __root__: List[str] = Field(..., description="List of commands to run")


class JksConfig(BaseModel):
    class Config:
        extra = Extra.forbid

    file_name: str = Field(
        ..., alias="fileName", description="Name of the JKS file to create"
    )
    passphrase: str = Field(
        ..., description="The passphrase to use to secure the jks and certificate"
    )


class CertbotAwsStoreCertificate(BaseModel):
    storage_path: str = Field(
        ...,
        description="path to folder to store the certbot certificates into.",
        regex="^/[\\x00-\\x7F]+$",
    )
    certificates_registry_table_name: Optional[str] = Field(
        None, description="Override the top level certificates_registry_table_name"
    )
    certificates_registry_table_region: Optional[str] = Field(
        None, description="Override the top level certificates_registry_table_region"
    )


class CertbotStore(BaseModel):
    certificates_registry_table_name: Optional[str] = Field(
        "certbot-registry", description="default table name to query for certificates."
    )
    certificates_registry_table_region: Optional[str] = Field(
        None,
        description="region to use for the certificates_registry_table_name. Defaults to current region found in profiles chain.",
    )
    certificates: Optional[Dict[str, CertbotAwsStoreCertificate]] = None


class Commands(BaseModel):
    class Config:
        extra = Extra.forbid

    post: Optional[CommandsDef] = Field(
        None, description="Commands to run after the file was retrieved"
    )
    pre: Optional[CommandsDef] = Field(
        None,
        description="Commands executed prior to the file being fetched, after `depends_on` completed",
    )


class SsmDef(BaseModel):
    parameter_name: Optional[str] = Field(None, alias="ParameterName")
    iam_override: Optional[IamOverrideDef] = Field(None, alias="IamOverride")


class SecretDef(BaseModel):
    secret_id: str = Field(..., alias="SecretId")
    version_id: Optional[str] = Field(None, alias="VersionId")
    version_stage: Optional[str] = Field(None, alias="VersionStage")
    json_key: Optional[str] = Field(
        None,
        alias="JsonKey",
        description="If the SecretString is a valid JSON, use the Key to map to the value stored in secret",
    )
    iam_override: Optional[IamOverrideDef] = Field(None, alias="IamOverride")


class S3Def1(BaseModel):
    s3_uri: Optional[S3Uri] = Field(None, alias="S3Uri")
    compose_x_uri: Optional[ComposeXUri] = Field(None, alias="ComposeXUri")
    bucket_name: Optional[str] = Field(
        None, alias="BucketName", description="Name of the S3 Bucket"
    )
    bucket_region: Optional[str] = Field(
        None,
        alias="BucketRegion",
        description="S3 Region to use. Default will ignore or retrieve via s3:GetBucketLocation",
    )
    key: Optional[str] = Field(
        None, alias="Key", description="Full path to the file to retrieve"
    )
    iam_override: Optional[IamOverrideDef] = Field(None, alias="IamOverride")


class S3Def(BaseModel):
    __root__: Union[S3Def1, Any, Any, Any]


class X509CertDef(BaseModel):
    class Config:
        extra = Extra.allow

    dir_path: Optional[str] = None
    email_address: Optional[EmailStr] = Field(
        "files-composer@compose-x.tld", alias="emailAddress"
    )
    common_name: Optional[
        constr(
            regex=r"^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]{0,61}[A-Za-z0-9])\Z"
        )
    ] = Field(None, alias="commonName")
    country_name: Optional[str] = Field("ZZ", alias="countryName", regex="^[A-Z]+$")
    locality_name: Optional[str] = Field("Anywhere", alias="localityName")
    state_or_province_name: Optional[str] = Field("Shire", alias="stateOrProvinceName")
    organization_name: Optional[str] = Field("NoOne", alias="organizationName")
    organization_unit_name: Optional[str] = Field(
        "Automation", alias="organizationUnitName"
    )
    validity_end_in_seconds: Optional[float] = Field(
        8035200,
        alias="validityEndInSeconds",
        description="Validity before cert expires, in seconds. Default 3*31*24*60*60=3Months",
    )
    key_file_name: str = Field(..., alias="keyFileName")
    cert_file_name: str = Field(..., alias="certFileName")
    group: Optional[str] = Field(
        "root",
        description="UNIX group name or GID owner of the file. Default to root(0)",
    )
    owner: Optional[str] = Field(
        "root", description="UNIX user or UID owner of the file. Default to root(0)"
    )
    jks_config: Optional[JksConfig] = Field(None, alias="jksConfig")


class Certificates(BaseModel):
    class Config:
        extra = Extra.forbid

    x509: Optional[Dict[str, X509CertDef]] = None


class SourceDef(BaseModel):
    url: Optional[UrlDef] = Field(None, alias="Url")
    ssm: Optional[SsmDef] = Field(None, alias="Ssm")
    s3: Optional[S3Def] = Field(None, alias="S3")
    secret: Optional[SecretDef] = Field(None, alias="Secret")


class FileDef(BaseModel):
    class Config:
        extra = Extra.allow

    path: Optional[str] = None
    content: Optional[str] = Field(
        None, description="The raw content of the file to use"
    )
    source: Optional[SourceDef] = None
    encoding: Optional[Encoding] = "plain"
    group: Optional[str] = Field(
        "root",
        description="UNIX group name or GID owner of the file. Default to root(0)",
    )
    owner: Optional[str] = Field(
        "root", description="UNIX user or UID owner of the file. Default to root(0)"
    )
    mode: Optional[str] = Field("0644", description="UNIX file mode")
    context: Optional[Context] = "plain"
    ignore_failure: Optional[Union[IgnoreFailureItem, bool]] = None
    commands: Optional[Commands] = None


class Model(BaseModel):
    files: Optional[Dict[str, FileDef]] = None
    certificates: Optional[Certificates] = None
    certbot_store: Optional[CertbotStore] = None
    iam_override: Optional[IamOverrideDef] = Field(None, alias="IamOverride")



# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2022 John Mille<john@compose-x.io>

"""
AWS Based filters
"""

from __future__ import annotations

from boto3.session import Session
from compose_x_common.compose_x_common import keyisset


def msk_bootstrap(msk_arn: str, broker_type: str) -> str:
    """
    Uses the ARN of a MSK cluster,
    and returns the list of BootStrap endpoints for a private MSK cluster using SASL IAM.
    If failed, returns the ARN.
    """
    session = Session()
    client = session.client("kafka")
    brokers_r = client.get_bootstrap_brokers(ClusterArn=msk_arn)
    if keyisset(broker_type, brokers_r):
        return brokers_r[broker_type]
    return msk_arn



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ecs_files_composer', 'ecs_files_composer.jinja2_filters']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'PyYAML>=6.0,<7.0',
 'boto3>=1.26,<2.0',
 'certbot-aws-store>=0.4.2.post0,<0.5.0',
 'compose-x-common>=1.2,<2.0',
 'flatdict>=4.0.1,<5.0.0',
 'jsonschema>=4.17,<5.0',
 'pyOpenSSL>=22,<23',
 'pydantic[email]>=1.9.0,<2.0.0',
 'pyjks>=20.0.0,<21.0.0',
 'requests>=2.27.1,<3.0.0']

entry_points = \
{'console_scripts': ['ecs_files_composer = ecs_files_composer.cli:main',
                     'files_composer = ecs_files_composer.cli:main']}

setup_kwargs = {
    'name': 'ecs-files-composer',
    'version': '1.2.1',
    'description': 'Files and configuration handler to inject configuration files into volumes for ECS containers',
    'long_description': ".. meta::\n    :description: ECS Files Composer input config\n    :keywords: AWS, AWS ECS, Docker, Compose, docker-compose, AWS S3, AWS SSM, Secrets, Configuration\n\n===================\nECS Files Composer\n===================\n\n\n.. image:: https://img.shields.io/pypi/v/ecs_files_composer.svg\n        :target: https://pypi.python.org/pypi/ecs_files_composer\n\n\n.. image:: https://codebuild.eu-west-1.amazonaws.com/badges?uuid=eyJlbmNyeXB0ZWREYXRhIjoiRWk3VUhxUi9peEdZRGs2cGFiTk5XM0VDK1FEQTBMN2JTdHh5b091NTVVdFd3RmpoM1lpdGkrTGtTZDJzVCt5dDBCc3Zsc0dXWHI5RHJRSG82UFJNdUJzPSIsIml2UGFyYW1ldGVyU3BlYyI6InJlYXlBWStNMDVZNEoyMnQiLCJtYXRlcmlhbFNldFNlcmlhbCI6MX0%3D&branch=main\n\n\nWhat does it do?\n=================\n\nECS Files Composer, although can be used in EKS and other Docker context, is a small program that will allow users\nto define files they need pulled out of AWS Services, such as AWS S3 or AWS SSM Parameter Store, and load the content\nto a given location, adding builtin capabilities to set file ownership, mode, and other handy features.\n\nThe configuration of the job to perform can be written in YAML or JSON (see examples), so long as they comply to a given\nschema.\n\nWhy use it?\n============\n\nHaving your core application, when reliant on configuration files, can be tricky to start in a way that the configuration\nneeds to be pulled first and then started. This can add un-necessary complexity and logic to the application.\nAnd some docker images you might pull off the shelves from DockerHub do not necessarily allow for configuration override\nfrom environment variables.\n\nBy using a sidecar that handles all of that logic, you delegate all of these activities to it. And with the ability to define\nwhich container to start first with success criteria, you also ensure that your application won't start without the configuration\nfiles it needs.\n\n.. hint::\n\n    This app / docker image can be used in any context, locally, on-premise, with Docker, on AWS ECS / EKS or in other cloud platforms.\n\nHow to use it ?\n=================\n\n`Full official documentation <https://docs.files-composer.compose-x.io/index.html>`__\n\n\nDocker\n----------------\n\n.. code-block:: bash\n\n    docker run public.ecr.aws/compose-x/ecs-files-composer:${VERSION:-latest} -h\n    docker run -v $PWD:/ /var/tmp,:/public.ecr.aws/compose-x/ecs-files-composer:${VERSION:-latest} -f files.yaml\n\n.. attention::\n\n    The default user is root to avoid running into issues when using chmod/chown and other commands.\n    Change behaviour at your own risks.\n\n\nCLI\n------------\n\n.. code-block:: bash\n\n\n    usage: ecs_files_composer [-h] [-f FILE_PATH | -e ENV_VAR | --from-ssm SSM_CONFIG | --from-s3 S3_CONFIG] [--role-arn ROLE_ARN] [_ ...]\n\n    optional arguments:\n      -h, --help            show this help message and exit\n      -f FILE_PATH, --from-file FILE_PATH\n                            Configuration for execution from a file\n      -e ENV_VAR, --from-env-var ENV_VAR\n                            Configuration for execution is in an environment variable\n      --from-ssm SSM_CONFIG\n                            Configuration for execution is in an SSM Parameter\n      --from-s3 S3_CONFIG   Configuration for execution is in an S3\n      --role-arn ROLE_ARN   The Role ARN to use for the configuration initialization\n",
    'author': 'John Preston',
    'author_email': 'john@compose-x.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
